package xyz.zhouxy.plusone.system.domain.model.account;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.LocalDateTime;
import java.util.Collection;
import java.util.Optional;

import javax.annotation.Nonnull;

import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.stereotype.Repository;

import cn.hutool.core.util.IdUtil;
import xyz.zhouxy.plusone.commons.domain.ValidatableStringRecord;
import xyz.zhouxy.plusone.commons.util.OptionalUtil;
import xyz.zhouxy.plusone.jdbc.JdbcRepositorySupport;

/**
 * AccountRepository 实现类
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Repository
public class AccountRepositoryImpl extends JdbcRepositorySupport<Account, Long> implements AccountRepository {

    private final AccountRoleRefDAO accountRoleDAO;

    public AccountRepositoryImpl(@Nonnull NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        super(namedParameterJdbcTemplate);
        this.accountRoleDAO = new AccountRoleRefDAO(namedParameterJdbcTemplate);
    }

    @Override
    protected final void doDelete(@Nonnull Account entity) {
        int i = update("""
                UPDATE sys_account SET deleted = id, "version" = "version" + 1
                WHERE id = :id AND deleted = 0 AND "version" = :version
                """,
                new MapSqlParameterSource()
                        .addValue("id", entity.getId().orElseThrow())
                        .addValue("version", entity.getVersion()));
        assertUpdateOneRow(i);
    }

    @Override
    protected final Optional<Account> doFindById(@Nonnull Long id) {
        return queryForObject("""
                SELECT
                    id, email, mobile_phone, username, "password", salt, avatar, sex, nickname, status,
                    created_by, updated_by, "version"
                FROM sys_account
                WHERE id = :id AND deleted = 0
                """,
                "id", id);
    }

    @Override
    public Optional<Account> findByEmail(Email email) {
        return queryForObject("""
                SELECT
                    id, email, mobile_phone, username, "password", salt, avatar, sex, nickname, status,
                    created_by, updated_by, "version"
                FROM sys_account
                WHERE email = :email AND deleted = 0
                """,
                "email", email.value());
    }

    @Override
    public Optional<Account> findByMobilePhone(MobilePhone mobilePhone) {
        return queryForObject("""
                SELECT
                    id, email, mobile_phone, username, "password", salt, avatar, sex, nickname, status,
                    created_by, updated_by, "version"
                FROM sys_account
                WHERE mobile_phone = :mobilePhone AND deleted = 0
                """,
                "mobilePhone", mobilePhone.value());
    }

    @Override
    public Optional<Account> findByUsername(Username username) {
        return queryForObject("""
                SELECT
                    id, email, mobile_phone, username, "password", salt, avatar, sex, nickname, status,
                    created_by, updated_by, "version"
                FROM sys_account
                WHERE username = :username AND deleted = 0
                """,
                "username", username.value());
    }

    @Override
    public boolean exists(Long id) {
        return queryForBool("SELECT EXISTS (SELECT 1 FROM sys_account WHERE id = :id AND deleted = 0 LIMIT 1)",
                "id", id);
    }

    @Override
    public boolean existsUsername(Username username) {
        return queryForBool(
                "SELECT EXISTS (SELECT 1 FROM sys_account WHERE username = :username AND deleted = 0 LIMIT 1)",
                "username", username.value());
    }

    @Override
    public boolean existsEmail(Email email) {
        return queryForBool("SELECT EXISTS (SELECT 1 FROM sys_account WHERE email = :email AND deleted = 0 LIMIT 1)",
                "email", email.value());
    }

    @Override
    public boolean existsMobilePhone(MobilePhone mobilePhone) {
        return queryForBool(
                "SELECT EXISTS (SELECT 1 FROM sys_account WHERE mobile_phone = :mobile_phone AND deleted = 0 LIMIT 1)",
                "mobile_phone", mobilePhone.value());
    }

    @Override
    public Collection<Account> findByRoleId(Long roleId) {
        return queryForList("""
                SELECT
                    a.id, a.email, a.mobile_phone, a.username, a."password", a.salt,
                    a.avatar, a.sex, a.nickname, a.status,
                    a.created_by, a.updated_by, a."version"
                FROM sys_account a
                LEFT JOIN sys_account_role ar ON a.id = ar.account_id
                WHERE ar.role_id = :roleId AND a.deleted = 0
                """,
                "roleId", roleId);
    }

    @Override
    protected final Account doInsert(@Nonnull Account entity) {
        long id = IdUtil.getSnowflakeNextId();
        int i = update("""
                INSERT INTO sys_account
                (id, email, mobile_phone, username, "password", salt, avatar, sex, nickname, status,
                 created_by, create_time)
                VALUES
                (:id, :email, :mobilePhone, :username, :password, :salt, :avatar, :sex, :nickname, :status,
                 :createdBy, :createTime)
                """,
                generateParamSource(id, entity));
        assertUpdateOneRow(i);
        this.accountRoleDAO.insertAccountRoleRefs(id, entity.getRoleIds());
        return find(id).orElseThrow();
    }

    @Override
    protected final Account doUpdate(@Nonnull Account entity) {
        int i = update("""
                UPDATE sys_account
                SET "email"        = :email,
                    "mobile_phone" = :mobilePhone,
                    "username"     = :username,
                    "password"     = :password,
                    "salt"         = :salt,
                    "avatar"       = :avatar,
                    "sex"          = :sex,
                    "nickname"     = :nickname,
                    "status"       = :status,
                    "updated_by"   = :updatedBy,
                    "update_time"  = :updateTime,
                    "version"      = "version" + 1
                WHERE id = :id AND deleted = 0 AND "version" = :version
                """,
                generateParamSource(entity));
        assertUpdateOneRow(i);
        this.accountRoleDAO.saveAccountRoleRefs(entity);
        return find(entity.getId().orElseThrow()).orElseThrow();
    }

    @Override
    protected final Account mapRow(ResultSet rs) throws SQLException {
        long accountId = rs.getLong("id");
        return new Account(
                accountId,
                rs.getString("username"),
                rs.getString("email"),
                rs.getString("mobile_phone"),
                Password.of(rs.getString("password"), rs.getString("salt")),
                AccountStatus.of(rs.getInt("status")),
                AccountInfo.builder()
                        .nickname(rs.getString("nickname"))
                        .avatar(rs.getString("avatar"))
                        .sex(Sex.of(rs.getInt("sex")))
                        .build(),
                this.accountRoleDAO.selectRoleIdsByAccountId(accountId),
                rs.getLong("created_by"),
                rs.getLong("updated_by"),
                rs.getLong("version"));
    }

    @Override
    protected final SqlParameterSource generateParamSource(Long id, @Nonnull Account entity) {
        final LocalDateTime now = LocalDateTime.now();
        final AccountInfo accountInfo = entity.getAccountInfo();
        final Optional<String> email = entity.getEmail().map(ValidatableStringRecord::value);
        final Optional<String> mobilePhone = entity.getMobilePhone().map(ValidatableStringRecord::value);
        final String username = entity.getUsername().value();
        final String password = entity.getPassword().value();
        final String salt = entity.getPassword().getSalt();
        final Optional<String> avatar = accountInfo.getAvatar().map(Object::toString);
        final int sex = accountInfo.getSex().getId();
        final Optional<String> nickname = accountInfo.getNickname().map(ValidatableStringRecord::value);
        final int status = entity.getStatus().getId();
        final Optional<Long> createdBy = entity.getCreatedBy();
        final Optional<Long> updatedBy = entity.getUpdatedBy();
        final long version = entity.getVersion();
        return new MapSqlParameterSource()
                .addValue("id", id)
                .addValue("email", OptionalUtil.orElseNull(email))
                .addValue("mobilePhone", OptionalUtil.orElseNull(mobilePhone))
                .addValue("username", username)
                .addValue("password", password)
                .addValue("salt", salt)
                .addValue("avatar", OptionalUtil.orElseNull(avatar))
                .addValue("sex", sex)
                .addValue("nickname", OptionalUtil.orElseNull(nickname))
                .addValue("status", status)
                .addValue("createdBy", OptionalUtil.orElseNull(createdBy))
                .addValue("createTime", now)
                .addValue("updatedBy", OptionalUtil.orElseNull(updatedBy))
                .addValue("updateTime", now)
                .addValue("version", version);
    }
}
